package de.uni_passau.fim.seibt.gitwrapper.repo;

import de.uni_passau.fim.seibt.gitwrapper.process.ProcessExecutor;

import java.util.Optional;
import java.util.function.Function;

public class Branch extends Reference {

    public Branch(Repository repo, String name) {
        super(repo, name);
    }

    /**
     * Checks this branch out, and tries to pull changes on this current branch.
     *
     * @return true if the pull was successful
     */
    public Boolean pull() {
        if (!repo.checkout(this)) {
            return false;
        }
        Optional<ProcessExecutor.ExecRes> fetch = git.exec(repo.getDir(), "pull");
        Function<ProcessExecutor.ExecRes, Boolean> toBoolean = res -> {
            boolean failed = git.failed(res);

            if (failed) {
                LOG.warning(() -> String.format("Pull of %s failed.", this));
            }

            return !failed;
        };

        return fetch.map(toBoolean).orElse(false);
    }

    /**
     * Returns the id of the {@link Commit} this reference is currently pointing to.
     *
     * @return the id of the {@link Commit} under this branch
     */
    @Override
    public String getId() {
        return repo.toHash(id).orElse(id);
    }

    /**
     * Reruns the branch name
     * @return the branch name
     */
    public String getName() {
        return id;
    }
}
